#ifndef TEST_WEBSITE_SRC_MODULE_JSON_LIBJSONINCPP_JSONOBJ_H
#define TEST_WEBSITE_SRC_MODULE_JSON_LIBJSONINCPP_JSONOBJ_H

#include "integer.h"
#include <vector>
#include <map>
#include <stdexcept>

namespace json {
    struct misuse : std::runtime_error {
        explicit misuse(const char *string);
    };

    enum json_t {
        null_symbol,
        false_symbol,
        true_symbol,
        integer,
        string,
        array,
        dictionary,
    };

    struct JSON;

    typedef std::vector<JSON> jarr;
    typedef std::map<std::string, JSON> jdict;

    struct JSON {
        void* value = NULL;
        json_t type = null_symbol;

        JSON() = default;
        explicit JSON(json_t type);
        explicit JSON(bool V);
        explicit JSON(int64_t val);
        explicit JSON(const Integer& V);
        explicit JSON(const char* str);
        explicit JSON(const std::string& V);
        explicit JSON(const std::vector<JSON>& V);
        explicit JSON(const std::map<std::string, JSON>& V);

        JSON(const JSON& other);
        JSON& operator=(const JSON& other);
        ~JSON();

        json_t getType() const;

        bool isNull() const;

        bool isBool() const;

        bool isInteger() const;

        bool isFalse() const;

        bool isTrue() const;

        bool isString() const;

        bool isArray() const;

        bool isDictionary() const;

        bool isNatalistic() const;

        bool isSymbol() const;

        bool toBool() const;

        const Integer& asInteger() const;

        const std::string& asString() const;

        const std::vector<JSON>& asArray() const;

        const std::map<std::string, JSON>& asDictionary() const;

        Integer& asInteger();

        std::string& asString();

        std::vector<JSON>& asArray();

        std::map<std::string, JSON>& asDictionary();

        JSON& operator[](size_t index);
        JSON& operator[](const std::string& key);

        const JSON& operator[](size_t index) const;
        const JSON& operator[](const std::string& key) const;

        bool operator==(const JSON& B) const;
        bool operator!=(const JSON& B) const;
    };
}

#endif
